#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*************************************************

 This solution uses inefficient name retrieval in the sense
 that names are looked up using sequential search.  It's obviously
 easy to modify the mechanism to do binary search or hashing, but
 with a limit of 300 names it's not necessary.

 The tree is built bottom up with all entries pointing to
 parents, no "down" pointers are maintained
 
*************************************************/



#define MAXNAMES  300
#define MAXCHARS  30
#define LAST_NAME_FLAG "no.child"

#define min(x,y) ((x) < (y) ? (x) : (y))
#define abs(x)   ((x) >= 0 ? (x) : -(x))

typedef struct t
{
    char *name;
    struct t *parent;
} treeNode;

/* allocate global character space */
static char buffer[(MAXCHARS+1)*MAXNAMES];
     
static  treeNode *names[MAXNAMES];
static char *last;
static int numNames;

static int ReadLine();
static int ProcessLines();
static treeNode *Install(char *);
static treeNode *Lookup(char *);
static int Ancestor(treeNode *,treeNode *);
static int CheckRelatedness(treeNode *,treeNode *);

int
main()
{
    numNames = 0;
    last = &buffer[0];

    while (ReadLine()){    /* process all pairs in input */
	;
    }

    while (ProcessLines()){   /* process all query pairs */
	;
    }
    exit(0);
}


static int
Ancestor(treeNode *t, treeNode *anchor)
/* return -1 if anchor is not a direct ancestor of t         */
/* return # of generations separating t and anchor otherwise */    
{
    int temp;
    
    if (t != (treeNode *) NULL){
	if (t == anchor)
	    return 0;
	else {
	    if ((temp = Ancestor(t->parent,anchor)) < 0)
		return -1;  /* not an ancestor */
	    else
		return 1 + temp;
	}
    }
    return -1;
}

static treeNode *
Lookup(char * s)
/* return pointer to node storing string s, or NULL if no such node */     
{
    int k;

    for(k=0;k<numNames;k++){
	if (strcmp(s,names[k]->name) == 0){
	    return names[k]; 	/* name already in list */
	}
    }
    return (treeNode *) NULL;
}


static treeNode *
Install(char *s)
/* return pointer to node storing s, create such a node if necessary */     
{
    treeNode *temp;
    if ((temp = Lookup(s)) == (treeNode *) NULL){
	strcpy(last,s);	         /* copy string into global string storage */
	
	names[numNames] = (treeNode *) malloc(sizeof(treeNode));
	names[numNames]->name = last;
	names[numNames]->parent = (treeNode *) NULL;
	
	last += (strlen(s) + 1);  /* update index into global string space */
	numNames++;
	return names[numNames-1];	
    }
    return temp;
}

static int
ReadLine()
/* read name pairs and install names in "family" structure */     
/* return 1 if not all names processed, 0 otherwise */
{
    char buf[BUFSIZ];
    int k;
    treeNode *child,*parent;
    
    if (gets(buf) != NULL){          /* get one line from input */
	for(k=0;buf[k] != ' ';k++)   /* find break of first/last names */
	    ;
	buf[k] = '\0';               /* first name terminated */
	for(k++;buf[k] == ' ';k++)   /* find begining of last name */
	    ;
	if (strcmp(buf,LAST_NAME_FLAG) == 0)
	    return 0;                /* don't install name flagging end */

	child = Install(buf);
	parent = Install(buf+k);     /* install names */
	
	child->parent = parent;      /* link names */

	return 1;
    }
    return 0;
}

static int
CheckRelatedness(treeNode *s, treeNode *t)
/* print relationship between s and t nodes if there is one */
/* return 1 if related, 0 otherwise */     
{
    int removed,k,gen = -1;
    int parent = 0,level = 0;

    /* check t and t's ancestors (parent*) as potential */
    /* ancestors/descendents of s                       */
    
    while (t != (treeNode *) NULL){
	if ((gen = Ancestor(s,t)) >= 0)     /* t is ancestor of s */
	    break;
	else if ((gen = Ancestor(t,s)) >=0){ 
	    parent = 1;
	    break;
	}
	t = t->parent;
	level++;
    }
    if (gen >= 0){
	if (level == 0){                /* direct ancestor/descendent */
	    for(k=3;k<=gen;k++)
		printf("great ");
	    if (gen > 1)
		printf("grand ");
	    if (gen == 0)
		printf("sibling\n");	/* I'm my own sibling */
	    else
		parent ? printf("parent\n") : printf("child\n");
	    return 1;
	}
	else{
	    k = min(gen,level);
	    if ((removed = abs(gen-level)) == 0 && (k-1) == 0)
		printf("sibling");
	    else{
		printf("%d cousin ",k-1);
		if (removed > 0)
		    printf("removed %d",removed);
	    }
	    printf("\n");			    
	    return 1;
	}
    }
    return 0;
}

static void
ProcessPair(char *s, char *t)
{
    treeNode *st, *tt;
    
    if ((st = Lookup(s)) == (treeNode *) NULL ||
	(tt = Lookup(t)) == (treeNode *) NULL)
	printf("no relation\n");
    else if (!CheckRelatedness(st,tt))
	printf("no relation\n");
}

static int
ProcessLines()
{
    char buf[BUFSIZ];
    int k;
    
    if (gets(buf) != NULL){
	for(k=0;buf[k] != ' ';k++)
	    ;
	buf[k] = '\0';
	for(k++;buf[k] == ' ';k++)
	    ;
	ProcessPair(buf,buf+k);
	return 1;
    }
    return 0;
}
